/*
BSD License

Copyright (c) 2013, Tom Everett
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of Tom Everett nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

grammar gff3;

document
    : '##gff-version 3\n' line+
    ;

line
    : commentline
    | dataline
    ;

dataline
    : seqid '\t' source '\t' type '\t' start '\t' end '\t' score '\t' strand '\t' phase '\t' attributes? EOL
    ;

attributes
    : attribute (';' attribute)*
    ;

attribute
    : TEXT '=' TEXT
    ;

seqid
    : TEXT
    ;
         
source
    : TEXT
    ;

type
    : TEXT
    ;

start
    : TEXT
    ;

end
    : TEXT
    ;

strand
    : TEXT
    ;

score
    : TEXT
    ;

phase
    : TEXT
    ;

commentline
    : COMMENTLINE
    ;   

COMMENTLINE
    : '#' .*? EOL
    ;

EOL
    : '\r'? '\n'
    ;

TEXT
    : (CHAR | SYMBOL | DIGIT)+
    ;

fragment CHAR
    : [a-zA-Z]
    ;

fragment DIGIT
    : [0-9]
    ;

fragment SYMBOL
    : '.'
    | ':'
    | '^'
    | '*'
    | '$'
    | '@'
    | '%'
    | '!'
    | '+'
    | '_'
    | '?'
    | '-'
    | '|'
    | ','
    | ' '
    ;